@@ -19,6 +19,22 @@ DelCallback = Callable[['Buf', int, str], None]
 InsCallback = Callable[['Buf', int], None]
 
 
+def _diff_codes(
+        a: list[str],
+        b: list[str],
+) -> Generator[tuple[str, int, int, int, int], None, None]:
+    matcher = difflib.SequenceMatcher(a=a, b=b)
+    for op, i1, i2, j1, j2 in reversed(matcher.get_opcodes()):
+        if op == 'replace':
+            if i2 - i1 == j2 - j1:
+                yield op, i1, i2, j1, j2
+            else:
+                yield 'delete', i1, i2, j1, j2
+                yield 'insert', i1, i1, j1, j2
+        elif op != 'equal':
+            yield op, i1, i2, j1, j2
+
+
 def _offsets(s: str, tab_size: int) -> tuple[int, ...]:
     ret = [0]
     for c in s:
@@ -135,8 +151,7 @@ class Buf:
         return victim
 
     def replace_lines(self, lines: list[str]) -> None:
-        matcher = difflib.SequenceMatcher(a=self._lines, b=lines)
-        for op, i1, i2, j1, j2 in reversed(matcher.get_opcodes()):
+        for op, i1, i2, j1, j2 in _diff_codes(self._lines, lines):
             if op == 'replace':
                 for i, j in zip(range(i1, i2), range(j1, j2)):
                     self[i] = lines[j]
@@ -146,8 +161,6 @@ class Buf:
             elif op == 'insert':
                 for j in reversed(range(j1, j2)):
                     self.insert(i1, lines[j])
-            elif op == 'equal':
-                pass
             else:
                 raise AssertionError(f'{op} {self._lines} {lines} ???')
 
@@ -267,9 +280,12 @@ class Buf:
         return y, x
 
     def fixup_position(self, dim: Dim) -> None:
-        self.y = min(self.y, len(self._lines) - 1)
+        self.assign_position(dim, x=self.x, y=self.y)
+
+    def assign_position(self, dim: Dim, *, x: int, y: int) -> None:
+        self.y = min(y, len(self._lines) - 1)
         self.scroll_screen_if_needed(dim)
-        self.x = min(self.x, len(self._lines[self.y]))
+        self.x = min(x, len(self._lines[self.y]))
 
     # rendered lines
 
@@ -302,7 +318,7 @@ class Buf:
 
     def _scroll_amount(self, dim: Dim) -> int:
         # integer round up without banker's rounding (so 1/2 => 1 instead of 0)
-        return int((dim.height + dim.y) / 2 + .5)
+        return int((dim.height + 1) / 2 + .5)
 
     def up(self, dim: Dim) -> None:
         if self.y > 0:
