@@ -1,12 +1,12 @@
+from __future__ import annotations
+
 import sys
 
-if sys.version_info >= (3, 8):  # pragma: no cover (>=py38)
+if sys.version_info >= (3, 8):  # pragma: >=3.8 cover
     from functools import cached_property
-else:  # pragma: no cover (<py38)
+else:  # pragma: <3.8 cover
     from typing import Callable
     from typing import Generic
-    from typing import Optional
-    from typing import Type
     from typing import TypeVar
 
     TSelf = TypeVar('TSelf')
@@ -18,8 +18,8 @@ else:  # pragma: no cover (<py38)
 
         def __get__(
                 self,
-                instance: Optional[TSelf],
-                owner: Optional[Type[TSelf]] = None,
+                instance: TSelf | None,
+                owner: type[TSelf] | None = None,
         ) -> TRet:
             assert instance is not None
             ret = instance.__dict__[self._func.__name__] = self._func(instance)
