@@ -24,6 +24,7 @@ from babi.buf import Buf
 from babi.buf import Modification
 from babi.dim import Dim
 from babi.hl.interface import FileHL
+from babi.hl.lint_errors import LintErrors
 from babi.hl.replace import Replace
 from babi.hl.selection import Selection
 from babi.hl.syntax import Syntax
@@ -229,6 +230,8 @@ class File:
         self.undo_stack: list[Action] = []
         self.redo_stack: list[Action] = []
         self._syntax = syntax
+        self._file_syntax = syntax.blank_file_highlighter()
+        self.lint_errors = LintErrors(syntax.color_manager, syntax.theme)
         self._trailing_whitespace = TrailingWhitespace(syntax.color_manager)
         self._replace_hl = Replace()
         self.selection = Selection()
@@ -272,23 +275,39 @@ class File:
 
         self.go_to_line(self.initial_line, dim)
 
+    @property
+    def root_scope(self) -> str:
+        return self._file_syntax.root_scope
+
     def _initialize_highlighters(self) -> None:
-        file_hls = []
         if self.filename is not None:
-            hl = self._syntax.file_highlighter(self.filename, self.buf[0])
-            file_hls.append(hl)
+            self._file_syntax = self._syntax.file_highlighter(
+                self.filename,
+                self.buf[0],
+            )
         else:
-            file_hls.append(self._syntax.blank_file_highlighter())
-        self._file_hls = (
-            *file_hls,
-            self._trailing_whitespace, self._replace_hl, self.selection,
+            self._file_syntax = self._syntax.blank_file_highlighter()
+
+        # hack due to https://github.com/python/mypy/issues/12360
+        file_hls: tuple[FileHL, ...] = (
+            self._file_syntax,
+            self.lint_errors,
+            self._trailing_whitespace,
+            self._replace_hl,
+            self.selection,
         )
+        self._file_hls = file_hls
+
         self.buf.clear_callbacks()
         for file_hl in self._file_hls:
             file_hl.register_callbacks(self.buf)
 
     def reload_theme(self, syntax: Syntax) -> None:
         self._syntax = syntax
+        self.lint_errors = self.lint_errors.clone(
+            syntax.color_manager,
+            syntax.theme,
+        )
         self._trailing_whitespace = TrailingWhitespace(syntax.color_manager)
         # only re-initialize the highlighters if we've loaded once
         if self._file_hls:
@@ -507,7 +526,10 @@ class File:
     def alt_up(self, dim: Dim) -> None:
         if self.buf.y > 0:
             offset = 1
-            while self.buf[self.buf.y - offset] == '':
+            while (
+                self.buf[self.buf.y - offset] == '' and
+                self.buf.y - offset - 1 >= 0
+            ):
                 offset += 1
             if offset >= 2:
                 self.buf.y -= offset
@@ -963,7 +985,6 @@ class File:
         to_display = min(self.buf.displayable_count, dim.height)
 
         for file_hl in self._file_hls:
-            # XXX: this will go away?
             file_hl.highlight_until(self.buf, self.buf.file_y + to_display)
 
         for i in range(to_display):
