@@ -27,8 +27,3 @@ class FileHL(Protocol):
     def regions(self) -> RegionsMapping: ...
     def highlight_until(self, lines: Buf, idx: int) -> None: ...
     def register_callbacks(self, buf: Buf) -> None: ...
-
-
-class HLFactory(Protocol):
-    def file_highlighter(self, filename: str, first_line: str) -> FileHL: ...
-    def blank_file_highlighter(self) -> FileHL: ...
