@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import curses
 
 from babi.cached_property import cached_property
@@ -34,7 +36,7 @@ def scrolled_line(s: str, x: int, width: int) -> str:
 
 class _CalcWidth:
     @cached_property
-    def _window(self) -> 'curses._CursesWindow':
+    def _window(self) -> curses._CursesWindow:
         return curses.newwin(1, 10)
 
     def wcwidth(self, c: str) -> int:
