@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import curses
 import functools
 import importlib.resources
 import json
@@ -9,8 +10,11 @@ from typing import NamedTuple
 
 from babi._types import Protocol
 from babi.color import Color
+from babi.color_manager import ColorManager
 from babi.fdict import FDict
 
+A_ITALIC = getattr(curses, 'A_ITALIC', 0x80000000)  # not always present
+
 
 class Style(NamedTuple):
     fg: Color | None
@@ -19,6 +23,15 @@ class Style(NamedTuple):
     i: bool
     u: bool
 
+    def attr(self, color_manager: ColorManager) -> int:
+        pair = color_manager.color_pair(self.fg, self.bg)
+        return (
+            curses.color_pair(pair) |
+            self.b * curses.A_BOLD |
+            self.i * A_ITALIC |
+            self.u * curses.A_UNDERLINE
+        )
+
     @classmethod
     def blank(cls) -> Style:
         return cls(fg=None, bg=None, b=False, i=False, u=False)
